.model small
.code

;Name    :Keysense.com                                                           
;Usage   :Passes Keypress' and shift state to batch files.                       
;        :Keysense with no parameters defaults to Keypress which reads any key   
;        :pressed including extended keys. Must be called again for second part  
;        :of errorlevel number remaining in keybuffer if extended keypressed.    
;        :The errorlevel value is equal to the ASCII value of the key pressed.
;        :Consult your manuals for ASCII values of keypresses.
;
;        :If An Invalid Parameter Is Passed, Keysense Defaults To None (A).
;                                                                                
;        :Parameters are: Alpabetical for Keyinput, no parameter same as A.      
;        :Ie: keysense (same as keysense a) ; keysense A ; keysense c.           
;        :Parameters may be typed in upper or lower case.
;                                                                                
;        :A-anykey, B-(A) with buffer flushed, C-anykey caps conversion,         
;        :D-(C) buffer flushed, E-alpha-num only with caps conversion,           
;        :F-(E) with buffer flushed.                                             
;                                                                                
;        :Parameters are: Numerical for Shiftstate                               
;        :Ie: keysense 1 ; keysense 3 ; keysense 0;2 ; keysense 341
;                                                                                
;        :Parameters may be grouped together, ie: keysense 1 3 5 2, to test for  
;        :combinations. The above example would return an errorlevel of 1 if the 
;        :Ins, Num, Alt, & Cap keys are all depressed. It would return a level   
;        :of 0 if any of those keys were not depressed. It disreguards any other 
;        :keys that might be depressed. If you wanted to test that those four    
;        :only are pressed, follow the test of those four with a test of the
;        :remaining four useing the 0 parameter. 0 folowed by key parameters 
;        :will test for keys not depressed. You could follow that test with
;        :keysense (individual parameters), to see what if any keys are pressed.
;        :Parameters may be separated by {;}, {,}, or spaces. They also may be
;        :chained, ie; keysense 3521 , keysense 52673 etc.
;                                                                                 
;        :9-Returns cumulative errorlevel, ie: 129 if ins key and r-shft pressed, 
;        :098 for caps,num,l-shft; 255 for all.                                   
;        :Parameter Number and Errorlevel values for each key is:                 
;                                                                                
;        :Para   Key     Errlvl                                                  
;        :                                                                       
;        : 1     Ins       128                                                   
;        : 2     Caps      64                                                    
;        : 3     Num       32                                                    
;        : 4     Scrl      16                                                    
;        : 5     Alt       8                                                     
;        : 6     Ctrl      4                                                     
;        : 7     L-Shft    2                                                     
;        : 8     R-Shft    1                                                     
;        : 9     cumul     total
;        : 0     Not pressed folowed by keys to test.
;
;        :Batchfile example for regular and extended keypress'.
;         
;        :entrky
;        keysense B
;        if errorlevel 0 if not errorlevel 1 goto highkey
;        if errorlevel 65 if not errorlevel 66 goto Apress
;        goto keyend
;        :highkey
;        keysense
;        if errorlevel 65 if not errorlevel 66 goto F7press
;        goto keyend
;        :Apress
;        echo The A key was pressed.
;        goto keyend
;        :F7press
;        echo The F7 key was pressed.
;        :keyend
;     
;        :Separate Blocks for extended keypress errorlevel testing so that the
;        :resulting action can be differentiated, ie: F7 and A both return 065
;        :as final errorlevel.
;
;        :Batchfile example for shiftstate
;
;         echo off
;         cls
;         echo Test.bat
;         echo Testing for keysense option
;         keysense 04
;         if errorlevel 0 if not errorlevel 1 goto speed
;         echo 
;         echo Scroll Key not depressed. Waiting for Response.
;         echo Press A key B key or Esc to exit.
;         :top
;         keysense f
;         if errorlevel 27 if not errorlevel 28 goto esc
;         if errorlevel 65 if not errorlevel 66 goto aky
;         if errorlevel 66 if not errorlevel 67 goto bky
;         echo No good. Not a key tested for.
;         goto top
;         :speed
;         echo 
;         echo Speed key.......Scroll is depressed. Skipped Options.
;         goto end
;         :aky
;         echo The A key Was Pressed.
;         goto end
;         :bky
;         echo The B key Was Pressed.
;         goto end
;         :esc
;         echo Esc. Pressed.
;         :end
;        
;

    mov     si,80h
    cld
    lodsb
    cmp     al,0Dh
    je      parm_E1
strngin:
    lodsb
    cmp     al,20h
    je      strngin
strngout:
    mov     dl,al      
    cmp     dl,30h 
    je      not_deprs
    cmp     dl,31h     
    jb      parm_E1
    cmp     dl,39h     
    ja      parm_tst
    jmp     Dpr_kys_dwn

parm_tst:
    cmp     dl,66h   
    ja      parm_E1 
    cmp     dl,42
    jb      parm_E1
    cmp     dl,46
    jna     Ent_parms

reduce:
    sub     dl,20h   
            
Ent_parms:
    mov     al,00h     
    cmp     dl,42h
    jz      parm_E2
    cmp     dl,43h
    jz      parm_E3
    cmp     dl,44h
    jz      parm_E4
    cmp     dl,45h 
    jz      parm_E5
    cmp     dl,46h
    jz      parm_E6

parm_E1:
    mov     ah,08h
    int     21h
    jmp     Ent_done

parm_E2:
    mov     al,08h
    mov     ah,0ch
    int     21h
    jmp     Ent_done

parm_E3:
    mov     ah,08h
    int     21h
    cmp     al,61h
    jb      Ent_done
    cmp     al,7ah
    ja      Ent_done
    sub     al,20h
    jmp     Ent_done

parm_E4:
    mov     al,08h
    mov     ah,0ch
    int     21h
    cmp     al,61h
    jb      Ent_done
    cmp     al,7ah
    ja      Ent_done
    sub     al,20h
    jmp     Ent_done

not_deprs:          
    mov    cx,3000h
    cld
    jmp     nxt_skip

parm_E5:
    mov     ah,08h
    int     21h
    cmp     al,1Bh
    je      Ent_done
    cmp     al,00h
    jnz     E5_Aj
    mov     ah,08h
    int     21h
    jmp     parm_E5

Dpr_kys_dwn:
    cmp     dl,39h 
    je      Any_Dpr_key
    jmp     do_again

E5_Aj:
    cmp     al,30h
    jb      parm_E5
    cmp     al,5ah
    ja      E5_Bj
    cmp     al,3ah
    jb      Ent_done
    cmp     al,41h
    jb      parm_E5
    jmp     Ent_done

E5_Bj:
    cmp     al,61h
    jb      parm_E5
    cmp     al,7ah
    ja      parm_E5
    sub     al,20h
    jmp     Ent_done

parm_E6:
    mov     al,08h
    mov     ah,0ch
    int     21h
    cmp     al,1Bh  
    je      Ent_done
    cmp     al,30h
    jb      parm_E6
    cmp     al,5ah
    ja      E6_Aj
    cmp     al,3ah
    jb      Ent_done
    cmp     al,41h
    jb      parm_E6

Ent_done:      
    mov     ah,4ch
    int     21h   

E6_Aj:
    cmp     al,61h
    jb      parm_E6
    cmp     al,7ah
    ja      parm_E6
    sub     al,20h
    jmp     Ent_done

nxt_skip:           
    lodsb           
    cmp     al,2ch  
    je      nxt_skip
    cmp     al,3bh  
    je      nxt_skip
    cmp     al,20h  
    je      nxt_skip
    jmp     do_again

Any_Dpr_key:
    mov     ah,02h
    int     16h
    mov     ah,4ch
    int     21h

do_again:    
    cmp     al,0dh
    jz      close_out
    cmp     al,2ch
    je      do_skip
    cmp     al,3bh
    je      do_skip
    cmp     al,20h  
    je      do_skip 
    mov     dl,al
    cmp     dl,30h
    jnb     yes_no
    mov     ax,4c02h
    int     21h

yes_no:
    sub     dl,30h
    cmp     dl,00h
    jz      Any_Dpr_key
    mov     cl,dl
    mov     dx,0000h
    mov     ah,02h
    int     16h
    mov     bl,al
    rcl     bl,cl
    adc     dl,00h
    cmp     ch,30h
    je      non_prsd
    cmp     dl,01h
    jmp     chk_prsd

non_prsd:
    cmp     dl,00h

chk_prsd:
    jz      its_good
    mov     ax,4c00h
    jmp     all_done

its_good:
    mov     bx,4c01h
    mov     ax,0000h

Do_skip:
    lodsb
    jmp     do_again

close_out:
    mov     ax,bx
    mov     ah,4ch

all_done:           
    int     21h
    
    end
